/* evilwm - Minimalist Window Manager for X
 * Copyright (C) 1999-2000 Ciaran Anscomb <kja3@atdot.org>
 * see README for license and other details. */

#include "evilwm.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <X11/cursorfont.h>

Display		*dpy;
Client		*current = NULL;
int		screen;
Window		root;
GC		invert_gc;
GC		string_gc;
GC		border_gc;
XFontStruct	*font;
int		iheight;
XColor		fg, bg;
Client		*head_client;
Atom		xa_wm_state;
Atom		xa_wm_change_state;
Atom		xa_wm_protos;
Atom		xa_wm_delete;
Atom		xa_wm_cmapwins;
Cursor		move_curs;
Cursor		resize_curs;
char		*opt_display = "";
char		*opt_font = DEF_FONT;
char		*opt_fg = DEF_FG;
char		*opt_bg = DEF_BG;
char		*opt_term;
int		opt_bw = DEF_BW;

int main(int argc, char *argv[])
{
	int i;

	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-fn") && i+1<argc)
			opt_font = argv[++i];
		else if (!strcmp(argv[i], "--display") && i+1<argc)
			opt_display = argv[++i];
		else if (!strcmp(argv[i], "--fg") && i+1<argc)
			opt_fg = argv[++i];
		else if (!strcmp(argv[i], "--bg") && i+1<argc)
			opt_bg = argv[++i];
//		else if (!strcmp(argv[i], "--bw") && i+1<argc)
//			opt_bw = atoi(argv[++i]);
		else if (!strcmp(argv[i], "--term") && i+1<argc) {
			opt_term = (char *)malloc(strlen(argv[++i])+1);
			strcpy(opt_term, argv[i]);
			strcat(opt_term, "&");
		} else {
			printf("usage: evilwm [--display display] [--term cmd]\n");
			printf("\t[--fg colour] [--bg colour]\n");
			exit(2);
		}
	}

	setup_display();
	scan_windows();
	do_event_loop();
	exit(1); /* just another brick in the -Wall... */
}

void setup_display()
{
	XGCValues gv;
	XSetWindowAttributes attr;
	XColor dummy;

	XSetErrorHandler(handle_xerror);
	signal(SIGTERM, quit_nicely);
	signal(SIGINT, quit_nicely);
	signal(SIGHUP, quit_nicely);

	dpy = XOpenDisplay(opt_display);

	if (!dpy) { 
		err("can't open display %s", opt_display);
		exit(1);
	}

	screen = DefaultScreen(dpy);
	root = RootWindow(dpy, screen);

	xa_wm_state = XInternAtom(dpy, "WM_STATE", False);
	xa_wm_change_state = XInternAtom(dpy, "WM_CHANGE_STATE", False);
	xa_wm_protos = XInternAtom(dpy, "WM_PROTOCOLS", False);
	xa_wm_delete = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
	xa_wm_cmapwins = XInternAtom(dpy, "WM_COLORMAP_WINDOWS", False);

	XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_fg, &fg, &dummy);
	XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_bg, &bg, &dummy);

	font = XLoadQueryFont(dpy, opt_font);
	if (!font) font = XLoadQueryFont(dpy, DEF_FONT);
	iheight = font->ascent + font->descent + 2*SPACE;

	move_curs = XCreateFontCursor(dpy, XC_fleur);
	resize_curs = XCreateFontCursor(dpy, XC_plus);

	gv.function = GXcopy;
	gv.foreground = fg.pixel;
	gv.font = font->fid;
	string_gc = XCreateGC(dpy, root, GCFunction | GCForeground | GCFont, &gv);

	gv.line_width = opt_bw;
	border_gc = XCreateGC(dpy, root, GCFunction | GCForeground | GCLineWidth, &gv);

	gv.function = GXinvert;
	gv.subwindow_mode = IncludeInferiors;
	invert_gc = XCreateGC(dpy, root, GCFunction | GCSubwindowMode | GCLineWidth | GCFont, &gv);

	attr.event_mask = ChildMask | ColormapChangeMask | ButtonMask | PropertyChangeMask;
	XChangeWindowAttributes(dpy, root, CWEventMask, &attr);
	// kja3
	//XGrabKey(dpy, AnyKey, ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_Return), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_Escape), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_y), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_u), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_b), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_n), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_h), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_j), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_k), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_l), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_KP_Insert), ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
	XGrabKey(dpy, XKeysymToKeycode(dpy, XK_Tab), Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
}

void scan_windows()
{
	unsigned int i, nwins;
	Window dw1, dw2, *wins;
	XWindowAttributes attr;

	XQueryTree(dpy, root, &dw1, &dw2, &wins, &nwins);
	for (i = 0; i < nwins; i++) {
		XGetWindowAttributes(dpy, wins[i], &attr);
		if (!attr.override_redirect && attr.map_state == IsViewable)
			make_new_client(wins[i]);
	}
	XFree(wins);
}
